   subroutine latin_random ( dim_num, point_num, seed, x )

      !*****************************************************************************80
      !
      !! LATIN_RANDOM returns points in a Latin Random square.
      !
      !  Discussion:
      !
      !    In each spatial dimension, there will be exactly one
      !    point whose coordinate value lies between consecutive
      !    values in the list:
      !
      !      ( 0, 1, 2, ..., point_num ) / point_num
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    08 April 2003
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    Input, integer ( kind = 4 ) DIM_NUM, the spatial dimension.
      !
      !    Input, integer ( kind = 4 ) POINT_NUM, the number of points.
      !
      !    Input/output, integer ( kind = 4 ) SEED, a seed for the random
      !    number generator.
      !
      !    Output, real ( kind = 8 ) X(DIM_NUM,POINT_NUM), the points.
      !
        implicit none

        integer ( kind = 4 ) dim_num
        integer ( kind = 4 ) point_num

        integer ( kind = 4 ) i
        integer ( kind = 4 ) j
        integer ( kind = 4 ) perm(point_num)
        integer ( kind = 4 ) seed
        real ( kind = 8 ) x(dim_num,point_num)
      !
      !  Pick DIM_NUM * POINT_NUM random numbers between 0 and 1.
      !
        call r8mat_uniform_01 ( dim_num, point_num, seed, x )
      !
      !  For spatial dimension I,
      !    pick a random permutation of 1 to POINT_NUM,
      !    force the corresponding I-th components of X to lie in the
      !    interval ( PERM(J)-1, PERM(J) ) / POINT_NUM.
      !
        do i = 1, dim_num

          call perm_uniform ( point_num, seed, perm )

          do j = 1, point_num
            x(i,j) = ( real ( perm(j) - 1, kind = 8 ) + x(i,j) ) &
                     / real ( point_num, kind = 8 )
          end do

        end do

        return
   end
   subroutine get_seed ( seed )

      !*****************************************************************************80
      !
      !! GET_SEED returns a seed for the random number generator.
      !
      !  Discussion:
      !
      !    The seed depends on the current time, and ought to be (slightly)
      !    different every millisecond.  Once the seed is obtained, a random
      !    number generator should be called a few times to further process
      !    the seed.
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    02 August 2004
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    Output, integer ( kind = 4 ) SEED, a pseudorandom seed value.
      !
        implicit none

        integer ( kind = 4 ) seed
        real ( kind = 8 ) temp
        character ( len = 10 ) time
        character ( len = 8 ) today
        integer ( kind = 4 ) values(8)
        character ( len = 5 ) zone

        call date_and_time ( today, time, zone, values )

        temp = 0.0D+00

        temp = temp + real ( values(2) - 1, kind = 8 ) /  11.0D+00
        temp = temp + real ( values(3) - 1, kind = 8 ) /  30.0D+00
        temp = temp + real ( values(5),     kind = 8 ) /  23.0D+00
        temp = temp + real ( values(6),     kind = 8 ) /  59.0D+00
        temp = temp + real ( values(7),     kind = 8 ) /  59.0D+00
        temp = temp + real ( values(8),     kind = 8 ) / 999.0D+00
        temp = temp                                    /   6.0D+00

        do while ( temp <= 0.0D+00 )
          temp = temp + 1.0D+00
        end do

        do while ( 1.0D+00 < temp )
          temp = temp - 1.0D+00
        end do

        seed = int ( real ( huge ( 1 ), kind = 8 ) * temp )
      !
      !  Never use a seed of 0 or maximum integer ( kind = 4 ).
      !
        if ( seed == 0 ) then
          seed = 1
        end if

        if ( seed == huge ( 1 ) ) then
          seed = seed - 1
        end if

        return
   end
   subroutine get_unit ( iunit )

      !*****************************************************************************80
      !
      !! GET_UNIT returns a free FORTRAN unit number.
      !
      !  Discussion:
      !
      !    A "free" FORTRAN unit number is a value between 1 and 99 which
      !    is not currently associated with an I/O device.  A free FORTRAN unit
      !    number is needed in order to open a file with the OPEN command.
      !
      !    If IUNIT = 0, then no free FORTRAN unit could be found, although
      !    all 99 units were checked (except for units 5, 6 and 9, which
      !    are commonly reserved for console I/O).
      !
      !    Otherwise, IUNIT is a value between 1 and 99, representing a
      !    free FORTRAN unit.  Note that GET_UNIT assumes that units 5 and 6
      !    are special, and will never return those values.
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    18 September 2005
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    Output, integer ( kind = 4 ) IUNIT, the free unit number.
      !
        implicit none

        integer ( kind = 4 ) i
        integer ( kind = 4 ) ios
        integer ( kind = 4 ) iunit
        logical lopen

        iunit = 0

        do i = 1, 99

          if ( i /= 5 .and. i /= 6 .and. i /= 9 ) then

            inquire ( unit = i, opened = lopen, iostat = ios )

            if ( ios == 0 ) then
              if ( .not. lopen ) then
                iunit = i
                return
              end if
            end if

          end if

        end do

        return
      end
      function i4_uniform_ab ( a, b, seed )

      !*****************************************************************************80
      !
      !! I4_UNIFORM_AB returns a scaled pseudorandom I4 between A and B.
      !
      !  Discussion:
      !
      !    An I4 is an integer ( kind = 4 ) value.
      !
      !    The pseudorandom number will be scaled to be uniformly distributed
      !    between A and B.
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license. 
      !
      !  Modified:
      !
      !    02 October 2012
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Reference:
      !
      !    Paul Bratley, Bennett Fox, Linus Schrage,
      !    A Guide to Simulation,
      !    Second Edition,
      !    Springer, 1987,
      !    ISBN: 0387964673,
      !    LC: QA76.9.C65.B73.
      !
      !    Bennett Fox,
      !    Algorithm 647:
      !    Implementation and Relative Efficiency of Quasirandom
      !    Sequence Generators,
      !    ACM Transactions on Mathematical Software,
      !    Volume 12, Number 4, December 1986, pages 362-376.
      !
      !    Pierre L'Ecuyer,
      !    Random Number Generation,
      !    in Handbook of Simulation,
      !    edited by Jerry Banks,
      !    Wiley, 1998,
      !    ISBN: 0471134031,
      !    LC: T57.62.H37.
      !
      !    Peter Lewis, Allen Goodman, James Miller,
      !    A Pseudo-Random Number Generator for the System/360,
      !    IBM Systems Journal,
      !    Volume 8, Number 2, 1969, pages 136-143.
      !
      !  Parameters:
      !
      !    Input, integer ( kind = 4 ) A, B, the limits of the interval.
      !
      !    Input/output, integer ( kind = 4 ) SEED, the "seed" value, which
      !    should NOT be 0.  On output, SEED has been updated.
      !
      !    Output, integer ( kind = 4 ) I4_UNIFORM_AB, a number between A and B.
      !
        implicit none

        integer ( kind = 4 ) a
        integer ( kind = 4 ) b
        integer ( kind = 4 ), parameter :: i4_huge = 2147483647
        integer ( kind = 4 ) i4_uniform_ab
        integer ( kind = 4 ) k
        real ( kind = 4 ) r
        integer ( kind = 4 ) seed
        integer ( kind = 4 ) value

        if ( seed == 0 ) then
          write ( *, '(a)' ) ' '
          write ( *, '(a)' ) 'I4_UNIFORM_AB - Fatal error!'
          write ( *, '(a)' ) '  Input value of SEED = 0.'
          stop 1
        end if

        k = seed / 127773

        seed = 16807 * ( seed - k * 127773 ) - k * 2836

        if ( seed < 0 ) then
          seed = seed + i4_huge
        end if

        r = real ( seed, kind = 4 ) * 4.656612875E-10
      !
      !  Scale R to lie between A-0.5 and B+0.5.
      !
        r = ( 1.0E+00 - r ) * ( real ( min ( a, b ), kind = 4 ) - 0.5E+00 ) & 
          +             r   * ( real ( max ( a, b ), kind = 4 ) + 0.5E+00 )
      !
      !  Use rounding to convert R to an integer between A and B.
      !
        value = nint ( r, kind = 4 )

        value = max ( value, min ( a, b ) )
        value = min ( value, max ( a, b ) )

        i4_uniform_ab = value

        return
   end
   subroutine perm_uniform ( n, seed, p )

      !*****************************************************************************80
      !
      !! PERM_UNIFORM selects a random permutation of N objects.
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    18 November 2008
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Reference:
      !
      !    Albert Nijenhuis, Herbert Wilf,
      !    Combinatorial Algorithms for Computers and Calculators,
      !    Academic Press, 1978,
      !    ISBN: 0-12-519260-6,
      !    LC: QA164.N54.
      !
      !  Parameters:
      !
      !    Input, integer ( kind = 4 ) N, the number of objects to be permuted.
      !
      !    Input/output, integer ( kind = 4 ) SEED, a seed for the random
      !    number generator.
      !
      !    Output, integer ( kind = 4 ) P(N), the permutation.  P(I) is the "new"
      !    location of the object originally at I.
      !
        implicit none

        integer ( kind = 4 ) n

        integer ( kind = 4 ) i
        integer ( kind = 4 ) i4_uniform_ab
        integer ( kind = 4 ) j
        integer ( kind = 4 ) k
        integer ( kind = 4 ) p(n)
        integer ( kind = 4 ) seed

        do i = 1, n
          p(i) = i
        end do

        do i = 1, n - 1
          j = i4_uniform_ab ( i, n, seed )
          k    = p(i)
          p(i) = p(j)
          p(j) = k
        end do

        return
   end
   subroutine r8mat_transpose_print ( m, n, a, title )

      !*****************************************************************************80
      !
      !! R8MAT_TRANSPOSE_PRINT prints an R8MAT, transposed.
      !
      !  Discussion:
      !
      !    An R8MAT is an MxN array of R8's, stored by (I,J) -> [I+J*M].
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    14 June 2004
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    Input, integer ( kind = 4 ) M, N, the number of rows and columns.
      !
      !    Input, real ( kind = 8 ) A(M,N), an M by N matrix to be printed.
      !
      !    Input, character ( len = * ) TITLE, a title.
      !
        implicit none

        integer ( kind = 4 ) m
        integer ( kind = 4 ) n

        real ( kind = 8 ) a(m,n)
        character ( len = * ) title

        call r8mat_transpose_print_some ( m, n, a, 1, 1, m, n, title )

        return
   end
   subroutine r8mat_transpose_print_some ( m, n, a, ilo, jlo, ihi, jhi, title )

      !*****************************************************************************80
      !
      !! R8MAT_TRANSPOSE_PRINT_SOME prints some of an R8MAT, transposed.
      !
      !  Discussion:
      !
      !    An R8MAT is an MxN array of R8's, stored by (I,J) -> [I+J*M].
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    10 September 2009
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    Input, integer ( kind = 4 ) M, N, the number of rows and columns.
      !
      !    Input, real ( kind = 8 ) A(M,N), an M by N matrix to be printed.
      !
      !    Input, integer ( kind = 4 ) ILO, JLO, the first row and column to print.
      !
      !    Input, integer ( kind = 4 ) IHI, JHI, the last row and column to print.
      !
      !    Input, character ( len = * ) TITLE, a title.
      !
        implicit none

        integer ( kind = 4 ), parameter :: incx = 5
        integer ( kind = 4 ) m
        integer ( kind = 4 ) n

        real ( kind = 8 ) a(m,n)
        character ( len = 14 ) ctemp(incx)
        integer ( kind = 4 ) i
        integer ( kind = 4 ) i2
        integer ( kind = 4 ) i2hi
        integer ( kind = 4 ) i2lo
        integer ( kind = 4 ) ihi
        integer ( kind = 4 ) ilo
        integer ( kind = 4 ) inc
        integer ( kind = 4 ) j
        integer ( kind = 4 ) j2hi
        integer ( kind = 4 ) j2lo
        integer ( kind = 4 ) jhi
        integer ( kind = 4 ) jlo
        character ( len = * ) title

        write ( *, '(a)' ) ' '
        write ( *, '(a)' ) trim ( title )

        if ( m <= 0 .or. n <= 0 ) then
          write ( *, '(a)' ) ' '
          write ( *, '(a)' ) '  (None)'
          return
        end if

        do i2lo = max ( ilo, 1 ), min ( ihi, m ), incx

          i2hi = i2lo + incx - 1
          i2hi = min ( i2hi, m )
          i2hi = min ( i2hi, ihi )

          inc = i2hi + 1 - i2lo

          write ( *, '(a)' ) ' '

          do i = i2lo, i2hi
            i2 = i + 1 - i2lo
            write ( ctemp(i2), '(i8,6x)' ) i
          end do

          write ( *, '(''  Row   '',5a14)' ) ctemp(1:inc)
          write ( *, '(a)' ) '  Col'
          write ( *, '(a)' ) ' '

          j2lo = max ( jlo, 1 )
          j2hi = min ( jhi, n )

          do j = j2lo, j2hi

            do i2 = 1, inc
              i = i2lo - 1 + i2
              write ( ctemp(i2), '(g14.6)' ) a(i,j)
            end do

            write ( *, '(i5,a,5a14)' ) j, ':', ( ctemp(i), i = 1, inc )

          end do

        end do

        return
   end
   subroutine r8mat_uniform_01 ( m, n, seed, r )

      !*****************************************************************************80
      !
      !! R8MAT_UNIFORM_01 fills an R8MAT with unit pseudorandom numbers.
      !
      !  Discussion:
      !
      !    An R8MAT is an MxN array of R8's, stored by (I,J) -> [I+J*M].
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    11 August 2004
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Reference:
      !
      !    Paul Bratley, Bennett Fox, Linus Schrage,
      !    A Guide to Simulation,
      !    Springer Verlag, pages 201-202, 1983.
      !
      !    Bennett Fox,
      !    Algorithm 647:
      !    Implementation and Relative Efficiency of Quasirandom
      !    Sequence Generators,
      !    ACM Transactions on Mathematical Software,
      !    Volume 12, Number 4, pages 362-376, 1986.
      !
      !    Peter Lewis, Allen Goodman, James Miller,
      !    A Pseudo-Random Number Generator for the System/360,
      !    IBM Systems Journal,
      !    Volume 8, pages 136-143, 1969.
      !
      !  Parameters:
      !
      !    Input, integer ( kind = 4 ) M, N, the number of rows and columns in
      !    the array.
      !
      !    Input/output, integer ( kind = 4 ) SEED, the "seed" value, which
      !    should NOT be 0.  On output, SEED has been updated.
      !
      !    Output, real ( kind = 8 ) R(M,N), the array of pseudorandom values.
      !
        implicit none

        integer ( kind = 4 ) m
        integer ( kind = 4 ) n

        integer ( kind = 4 ) i
        integer ( kind = 4 ), parameter :: i4_huge = 2147483647
        integer ( kind = 4 ) j
        integer ( kind = 4 ) k
        integer ( kind = 4 ) seed
        real ( kind = 8 ) r(m,n)

        do j = 1, n

          do i = 1, m

            k = seed / 127773

            seed = 16807 * ( seed - k * 127773 ) - k * 2836

            if ( seed < 0 ) then
              seed = seed + i4_huge
            end if

            r(i,j) = real ( seed, kind = 8 ) * 4.656612875D-10

          end do
        end do

        return
   end
   subroutine r8mat_write ( output_filename, m, n, table )

      !*****************************************************************************80
      !
      !! R8MAT_WRITE writes an R8MAT file.
      !
      !  Discussion:
      !
      !    An R8MAT is an array of R8 values.
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    31 May 2009
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    Input, character ( len = * ) OUTPUT_FILENAME, the output file name.
      !
      !    Input, integer ( kind = 4 ) M, the spatial dimension.
      !
      !    Input, integer ( kind = 4 ) N, the number of points.
      !
      !    Input, real ( kind = 8 ) TABLE(M,N), the table data.
      !
        implicit none

        integer ( kind = 4 ) m
        integer ( kind = 4 ) n

        integer ( kind = 4 ) j
        character ( len = * ) output_filename
        integer ( kind = 4 ) output_status
        integer ( kind = 4 ) output_unit
        character ( len = 30 ) string
        real ( kind = 8 ) table(m,n)
      !
      !  Open the file.
      !
        call get_unit ( output_unit )

        open ( unit = output_unit, file = output_filename, &
          status = 'replace', iostat = output_status )

        if ( output_status /= 0 ) then
          write ( *, '(a)' ) ' '
          write ( *, '(a)' ) 'R8MAT_WRITE - Fatal error!'
          write ( *, '(a,i8)' ) '  Could not open the output file "' // &
            trim ( output_filename ) // '" on unit ', output_unit
          output_unit = -1
          stop
        end if
      !
      !  Create a format string.
      !
      !  For less precision in the output file, try:
      !
      !                                            '(', m, 'g', 14, '.', 6, ')'
      !
        if ( 0 < m .and. 0 < n ) then

          write ( string, '(a1,i8,a1,i8,a1,i8,a1)' ) '(', m, 'g', 24, '.', 16, ')'
      !
      !  Write the data.
      !
          do j = 1, n
            write ( output_unit, string ) table(1:m,j)
          end do

        end if
      !
      !  Close the file.
      !
        close ( unit = output_unit )

        return
   end
   subroutine timestamp ( )

      !*****************************************************************************80
      !
      !! TIMESTAMP prints the current YMDHMS date as a time stamp.
      !
      !  Example:
      !
      !    31 May 2001   9:45:54.872 AM
      !
      !  Licensing:
      !
      !    This code is distributed under the GNU LGPL license.
      !
      !  Modified:
      !
      !    18 May 2013
      !
      !  Author:
      !
      !    John Burkardt
      !
      !  Parameters:
      !
      !    None
      !
        implicit none

        character ( len = 8 ) ampm
        integer ( kind = 4 ) d
        integer ( kind = 4 ) h
        integer ( kind = 4 ) m
        integer ( kind = 4 ) mm
        character ( len = 9 ), parameter, dimension(12) :: month = (/ &
          'January  ', 'February ', 'March    ', 'April    ', &
          'May      ', 'June     ', 'July     ', 'August   ', &
          'September', 'October  ', 'November ', 'December ' /)
        integer ( kind = 4 ) n
        integer ( kind = 4 ) s
        integer ( kind = 4 ) values(8)
        integer ( kind = 4 ) y

        call date_and_time ( values = values )

        y = values(1)
        m = values(2)
        d = values(3)
        h = values(5)
        n = values(6)
        s = values(7)
        mm = values(8)

        if ( h < 12 ) then
          ampm = 'AM'
        else if ( h == 12 ) then
          if ( n == 0 .and. s == 0 ) then
            ampm = 'Noon'
          else
            ampm = 'PM'
          end if
        else
          h = h - 12
          if ( h < 12 ) then
            ampm = 'PM'
          else if ( h == 12 ) then
            if ( n == 0 .and. s == 0 ) then
              ampm = 'Midnight'
            else
              ampm = 'AM'
            end if
          end if
        end if

        write ( *, '(i2.2,1x,a,1x,i4,2x,i2,a1,i2.2,a1,i2.2,a1,i3.3,1x,a)' ) &
          d, trim ( month(m) ), y, h, ':', n, ':', s, '.', mm, trim ( ampm )

        return
   end
